function [headingEst] = estimateHeadingRL(stimulus,plotMode,iters)
% Copyright (C) 2016-2020 Charlie S. Burlingham
%
% anisotropic threhsolding turned off for now, as it was breaking their method. I believe this step is an issue in general for relatively sparse flow fields.
% delta = .01; % anisotropy threshold (ratio of min/max eigenvalues of candidate diff vecs for each r_i). delta = .01 from Li & Warren 2000 simulations
%
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.


%% create flow field

for gg = 1:iters
     
     
     % generate optic flow corresponding to phase motion stimulus (last flow field only)
     for i = 1:stimulus.nPlanes
          [stimulus.dots_m{i}, stimulus.dots_deg{i}] = make_dot_plane(stimulus.dot_density(i), stimulus.plane_dist(i), stimulus.view_window, stimulus.exclude);
          stimulus.velocity_field{i} = calculate_plane_flow_timestamps(stimulus.dots_m{i}, stimulus.dots_deg{i}, stimulus.translate, stimulus.rotate.*pi./180, stimulus.view_dist, stimulus.nFlowFields-1, stimulus.plane_dist(i), stimulus.nFlowFields, stimulus.duration);  % must multiply angular rotation by pi/180 to put in equiv. units to envelope motion condition!
     end
     
     if plotMode
          figure(1); quiver(stimulus.dots_deg{1}(1,:),stimulus.dots_deg{1}(2,:),stimulus.velocity_field{1}(1,:),stimulus.velocity_field{1}(2,:),0)
          hold on; quiver(stimulus.dots_deg{2}(1,:),stimulus.dots_deg{2}(2,:),stimulus.velocity_field{2}(1,:),stimulus.velocity_field{2}(2,:),0)
          title('Velocity field')
     end
     
     % ensure both planes have exact same number of elements!
     [minVal,minind] = min([size(stimulus.dots_deg{1},2) size(stimulus.dots_deg{2},2)]);
     [maxVal,maxind] = max([size(stimulus.dots_deg{1},2) size(stimulus.dots_deg{2},2)]);
     stimulus.dots_deg{maxind}(:,minVal+1:maxVal) = [];
     stimulus.velocity_field{maxind}(:,minVal+1:maxVal) = [];
     
     
     % format data
     flow1.X = [stimulus.dots_deg{1}(1,:)'; stimulus.dots_deg{2}(1,:)'];
     flow1.Y = [stimulus.dots_deg{1}(2,:)'; stimulus.dots_deg{2}(2,:)'];
     flow1.Dx = [stimulus.velocity_field{1}(1,:)'; stimulus.velocity_field{2}(1,:)'];
     flow1.Dy = [stimulus.velocity_field{1}(2,:)'; stimulus.velocity_field{2}(2,:)'];
     
     flow1_degPerF.Dx = (flow1.Dx.*stimulus.duration)./stimulus.nFlowFields;
     flow1_degPerF.Dy = (flow1.Dy.*stimulus.duration)./stimulus.nFlowFields;
     flow1_degPerF.X = flow1.X;
     flow1_degPerF.Y = flow1.Y;
     
     
     %% Now reformat flow and position data.
     
     len =  length(flow1_degPerF.X);
     pos1 = [flow1_degPerF.X flow1_degPerF.Y]; % all pos. r_i or r_j
     f1 = [flow1_degPerF.Dx flow1_degPerF.Dy]; % all flow vector. u_i or u_j
     
     %% compute closest pts between the two planes
     
     clear closest; clear bestFitLineTLS;
     for ii = 1:size(pos1,1)
          
          distances = sqrt(sum(bsxfun(@minus, pos1, pos1(ii,:)).^2,2)); % L2 norm like Rieger & Lawton 1985
          closest{ii} = find(distances<stimulus.e); % either closest vectors in general in the flow field... or closest vector in plane 2 to plane 1 in order of plane 1 pos vector
           
          diffVecs{ii} = bsxfun(@minus, f1(ii,:), f1(closest{ii},:)); %  delta_ij U in eq. 5
          endPts = bsxfun(@plus, pos1(ii,:), diffVecs{ii}); % r_i + delta_ij U
          
          % Thresholding: if there are less than 3 nearby vectors or anisotropy is too low, skip to next iteration of loop and mark this one as trash
          %[v d] = eig(diffVecs{ii}'*diffVecs{ii});
          
          if size(endPts,1) < 3 %|| (d(1)/d(4)) > delta    % anistropy: ratio of max and min eigenvalues . % comment this and line 39 & 96 in to use the thresholding again
               bestFitLineTLS(ii,:) = [NaN NaN];
               continue
          end
          
          % to plot the difference vectors for r_i
          %figure; line([repmat(pos1(ii,1),[size(endPts,1),1])'; endPts(:,1)'],[repmat(pos1(ii,2),[size(endPts,1),1])'; endPts(:,2)']);
          
          % plot the difference vectors with the eigenvectors corr to
          % the max and min eigevalues
          % scaledEigs = [v(:,1).*d(1) v(:,2).*d(4)]
          % figure; plotv(diffVecs{ii}'); hold on; plotv(v','k'); axis equal
          
          
          
          % Now do TLS regression (Deming) to find the best fit line
          % thru r_i that minimizes the projections onto these endPts.
          % If the intercept is fixed (i.e., data centered around
          % 0,0), then the solution is the first eigenvector of the
          % covariance matrix.
          
          bestFitLineTLS(ii,:) = pca(diffVecs{ii}'*diffVecs{ii})' + pos1(ii,:);
          
          
          if plotMode
               figure(2)
               line([pos1(ii,1) bestFitLineTLS(ii,1)],[pos1(ii,2) bestFitLineTLS(ii,2)])
               hold on; plot(pos1(ii,1),pos1(ii,2),'ro')
          end
     end
     
     
     % now delete all pos  & difference vectors that weren't properly constrained
     % (i.e., the ones that didn't have at at least 3 nearby vectors)
     trash = find(isnan(bestFitLineTLS(:,1)));
     bestFitLineTLS(trash,:) = [];
     pos1(trash,:) = [];
     
     
     
     %% estimate singularity using least squares intersection of N lines - to estimate heading
     
     % create lines
     p1 = pos1(:,1);
     q1 = bestFitLineTLS(:,1);
     p2 = pos1(:,2);
     q2 = bestFitLineTLS(:,2);
     
     lines1 = [p1 q1];
     lines2 = [p2 q2];
     
     % plot lines
     if plotMode
          figure(3);line([p1 q1]',[p2 q2]')
          title('Lines')
     end
     
     headingEst(gg,:) = lineXline([p1 p2],[q1 q2]) % estimate singularity in deg
     
     % plot singularity
     if plotMode
          hold on
          figure(1); plot(headingEst(gg,1),headingEst(gg,2),'go');
          figure(2); plot(headingEst(gg,1),headingEst(gg,2),'go');
          figure(3); plot(headingEst(gg,1),headingEst(gg,2),'go');
     end
     
end