function [meanSaccPerTrialVec, meanEyeTraceH, meanEyeTraceV] = wsEyeAnal(directory)
% Charlie S. Burlingham
% Nov 23, 2020
%
%
% Copyright (C) 2016-2020 Charlie S. Burlingham
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

close all; clc;

if ieNotDefined('directory'); directory = pwd; end

% only load files that are NOT hidden! and are .mat files
s = dir(directory); % 'directory' is the investigated directory
%remove hidden files
s2 = s(arrayfun(@(x) ~strcmp(x.name(1),'.'),s));

arrayStringS2 = {s2(:).name};
stimfiles = arrayStringS2(endsWith(arrayStringS2,'.mat'));

byRunEyeData = [];
for ii = 1:length(stimfiles)
    try % error handling for when the mat file has been renamed, temp fix... should really rename all mat files and edf files that I renamed earlier (bc they aren't linked properly anymore)
        byRunEyeData = [byRunEyeData runEyeAnal(stimfiles{ii})];
    catch
    end
end

rotationVels = [-2 -0.8 0 .8 2];

for rr = 1:length(rotationVels)
    for jj = 1:length(byRunEyeData) % creates vector of mean number of saccades per trial per rotation condition over each run for that subject
        if mod(jj,5) == 0; disp(jj);end
        meanSaccPerTrialVec(rr,jj) = mean(byRunEyeData(jj).avgSaccs{1, rr}); % rr 1:5 -> Left fast, Left slow, PT, Right slow, Right fast
        
        meanEyeTraceH(:,jj,rr) = byRunEyeData(jj).eyeSignal{1, rr}(:,1);
        
        meanEyeTraceV(:,jj,rr) = byRunEyeData(jj).eyeSignal{1, rr}(:,2);
    end
end

%% now you can plot the mean number of saccades (over all trials) per rotation velocity
figure; plot(rotationVels,mean(meanSaccPerTrialVec,2));

%% and can also see if the observers maintained fixation by looking at the mean eye traces
figure; plot(rotationVels,squeeze(mean(meanEyeTraceH,2)));
hold on;
plot(rotationVels,squeeze(mean(meanEyeTraceV,2)));

end
