function [dots_deg,still_legal] = project_dot_world2(dots_m, view_window, view_dist)
% Author: Charlie S. Burlingham
% Last Updated: Nov 23, 2020
%
% Takes the 3D world of dots, and projects it into two dimensions,
% according to the observer's distance from the screen and the size of the
% viewing window.
% 
% dots_m is a 3 row vector containing the [X,Y,Z] positions of the dots, in 
%       m.
% 
% view_window is the size [X,Y] of the viewing window for the projected
%       dots, in dva.
% 
% view_dist is the observer's distance from the screen, in m.
%
% dots_dog is a two row vector of the [X;Y] of each visible dot, in deg.
%
%
% Copyright (C) 2016-2020 Charlie S. Burlingham
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

dots_2d_m = [view_dist*dots_m(1,:)./dots_m(3,:);view_dist*dots_m(2,:)./dots_m(3,:)];
dots_deg = 2*atand((dots_2d_m/2)/view_dist); %convert coordinates from m to dva

still_legalX=abs(dots_deg(1,:))<=(view_window(1)/2);
still_legalY=abs(dots_deg(2,:))<=(view_window(2)/2);
still_legalW=dots_m(3,:)>=view_dist;
still_legal = find(still_legalX.*still_legalY.*still_legalW);

dots_m(:,dots_m(3,:)<view_dist)=[]; %clip out the dots that are closer than the window
dots_deg = 2*atand((dots_2d_m/2)/view_dist); %convert coordinates from m to dva
